@@ -106,11 +106,11 @@ class Dependency(object):
 
         if not self.component:
             # Translators: %s is a Python module name or another os component
-            message = _("- %s not found on the system" % self.modulename)
+            message = _("- %s not found on the system") % self.modulename
         else:
             # Translators: %s is a Python module name or another os component
-            message = _("- %s version %s is installed but Pitivi requires at least version %s" % (
-                self.modulename, self.version_installed, self.version_required_string))
+            message = _("- %s version %s is installed but Pitivi requires at least version %s") % (
+                self.modulename, self.version_installed, self.version_required_string)
 
         if self.additional_message is not None:
             message += "\n    -> " + self.additional_message
@@ -202,43 +202,53 @@ def _check_audiosinks():
     from gi.repository import Gst
     # Yes, this can still fail, if PulseAudio is non-responsive for example.
     sink = Gst.ElementFactory.make("autoaudiosink", None)
-    if not sink:
+    return sink
+
+
+def _using_broadway_display():
+    from gi.repository import Gdk
+    from gi.repository import GObject
+    try:
+        gdk_broadway_display_type = GObject.type_from_name("GdkBroadwayDisplay")
+    except RuntimeError:
         return False
-    return True
+    display = Gdk.Display.get_default()
+    return GObject.type_is_a(display.__gtype__, gdk_broadway_display_type)
 
 
 def _check_videosink():
     from gi.repository import Gst
-    from gi.repository import Gdk
-    from gi.repository import GObject
     global videosink_factory
 
-    try:
-        # If using GdkBroadwayDisplay make sure not to try to use gtkglsink
-        # as it would segfault right away.
-        if GObject.type_is_a(Gdk.Display.get_default().__gtype__,
-                             GObject.type_from_name("GdkBroadwayDisplay")):
-            videosink_factory = Gst.ElementFactory.find("gtksink")
-            return True
-    except RuntimeError:
-        pass
-
-    if "gtkglsink" in os.environ.get("PITIVI_UNSTABLE_FEATURES", ''):
+    # If using GdkBroadwayDisplay make sure not to try to use gtkglsink
+    # as it would segfault right away.
+    if not videosink_factory and \
+            not _using_broadway_display() and \
+            "gtkglsink" in os.environ.get("PITIVI_UNSTABLE_FEATURES", ""):
         sink = Gst.ElementFactory.make("gtkglsink", None)
-        if not sink:
-            videosink_factory = sink.get_factory()
-        elif sink.set_state(Gst.State.READY) == Gst.StateChangeReturn.SUCCESS:
-            videosink_factory = sink.get_factory()
-            sink.set_state(Gst.State.NULL)
-        else:
-            videosink_factory = Gst.ElementFactory.find("gtksink")
-    else:
+        if sink:
+            res = sink.set_state(Gst.State.READY)
+            if res == Gst.StateChangeReturn.SUCCESS:
+                videosink_factory = sink.get_factory()
+                sink.set_state(Gst.State.NULL)
+
+    if not videosink_factory:
         videosink_factory = Gst.ElementFactory.find("gtksink")
 
-    if videosink_factory:
-        return True
+    return videosink_factory
 
-    return False
+
+def _check_vaapi():
+    from gi.repository import Gst
+    if "vaapi" in os.environ.get("PITIVI_UNSTABLE_FEATURES", ""):
+        print("Vaapi decoders enabled.")
+        return
+
+    for feature in Gst.Registry.get().get_feature_list_by_plugin("vaapi"):
+        if isinstance(feature, Gst.ElementFactory):
+            klass = feature.get_klass()
+            if "Decoder" in klass and "Video" in klass:
+                feature.set_rank(Gst.Rank.MARGINAL)
 
 
 def _check_gst_python():
@@ -277,27 +287,29 @@ def check_requirements():
         dependency.check()
         if not dependency.satisfied:
             missing_soft_deps[dependency.modulename] = dependency
-            print((_("Missing soft dependency:")))
+            print(_("Missing soft dependency:"))
             print(dependency)
 
     if not hard_dependencies_satisfied:
         return False
 
     if not _check_gst_python():
-        print((_("ERROR — Could not create a Gst.Fraction — "
-              "this means gst-python is not installed correctly.")))
+        print(_("ERROR — Could not create a Gst.Fraction — "
+                "this means gst-python is not installed correctly."))
         return False
 
     if not _check_audiosinks():
-        print((_("Could not create audio output sink. "
-                 "Make sure you have a valid one (pulsesink, alsasink or osssink).")))
+        print(_("Could not create audio output sink. "
+                "Make sure you have a valid one (pulsesink, alsasink or osssink)."))
         return False
 
     if not _check_videosink():
-        print((_("Could not create video output sink. "
-                 "Make sure you have a gtksink available.")))
+        print(_("Could not create video output sink. "
+                "Make sure you have a gtksink available."))
         return False
 
+    _check_vaapi()
+
     return True
 
 
@@ -307,9 +319,8 @@ def require_version(modulename, version):
     try:
         gi.require_version(modulename, version)
     except ValueError:
-        print((_("Could not import '%s'. "
-                 "Make sure you have it available."
-                 % modulename)))
+        print(_("Could not import '%s'. Make sure you have it available.")
+              % modulename)
         exit(1)
 
 
@@ -322,8 +333,8 @@ def initialize_modules():
     try:
         import gi
     except ImportError:
-        print((_("Could not import 'gi'. "
-                 "Make sure you have pygobject available.")))
+        print(_("Could not import 'gi'. "
+                "Make sure you have pygobject available."))
         exit(1)
 
     require_version("Gtk", GTK_API_VERSION)
@@ -337,10 +348,14 @@ def initialize_modules():
 
     require_version("Gst", GST_API_VERSION)
     require_version("GstController", GST_API_VERSION)
+    require_version("GstTranscoder", GST_API_VERSION)
     from gi.repository import Gst
     from pitivi.configure import get_audiopresets_dir, get_videopresets_dir
     Gst.init(None)
 
+    if not os.environ.get("GES_DISCOVERY_TIMEOUT"):
+        os.environ["GES_DISCOVERY_TIMEOUT"] = "5"
+
     require_version("GES", GST_API_VERSION)
     from gi.repository import GES
     res, sys.argv = GES.init_check(sys.argv)
@@ -372,28 +387,31 @@ def initialize_modules():
 # a specific version requirement, they have the "None" value.
 
 GST_API_VERSION = "1.0"
+GST_VERSION = "1.12.3"
 GTK_API_VERSION = "3.0"
 GLIB_API_VERSION = "2.0"
 HARD_DEPENDENCIES = [GICheck("3.20.0"),
                      CairoDependency("1.10.0"),
-                     GstDependency("Gst", GST_API_VERSION, "1.10.2"),
-                     GstDependency("GES", GST_API_VERSION, "1.10.2"),
+                     GstDependency("Gst", GST_API_VERSION, GST_VERSION),
+                     GstDependency("GES", GST_API_VERSION, GST_VERSION),
                      GIDependency("GstTranscoder", GST_API_VERSION),
+                     GIDependency("GstVideo", GST_API_VERSION),
                      GtkDependency("Gtk", GTK_API_VERSION, "3.20.0"),
                      ClassicDependency("numpy"),
                      GIDependency("Gio", "2.0"),
-                     # TODO Remove extra_modules when we depend
-                     # on Gst 1.12
-                     GstPluginDependency("gtk", "1.10.2",
-                                         extra_modulenames=["gstgtk"]),
-                     GstPluginDependency("gdkpixbuf", "1.10.2"),
+                     GstPluginDependency("gtk", GST_VERSION),
+                     GstPluginDependency("gdkpixbuf", GST_VERSION),
                      ClassicDependency("matplotlib"),
                      ]
 
 SOFT_DEPENDENCIES = (
-    ClassicDependency("pycanberra", None, _("enables sound notifications when rendering is complete")),
-    GIDependency("Notify", "0.7", None, _("enables visual notifications when rendering is complete")),
-    GstPluginDependency("libav", None, _("additional multimedia codecs through the GStreamer Libav library")),
-    GstPluginDependency("debugutilsbad", None, _("enables a watchdog in the GStreamer pipeline."
-                                                 " Use to detect errors happening in GStreamer"
-                                                 " and recover from them")))
+    GIDependency("GSound", "1.0", None,
+                 _("enables sound notifications when rendering is complete")),
+    GIDependency("Notify", "0.7", None,
+                 _("enables visual notifications when rendering is complete")),
+    GstPluginDependency("libav", None,
+                        _("additional multimedia codecs through the GStreamer Libav library")),
+    GstPluginDependency("debugutilsbad", None,
+                        _("enables a watchdog in the GStreamer pipeline."
+                          " Use to detect errors happening in GStreamer"
+                          " and recover from them")))
