@@ -1,7 +1,5 @@
+# -*- coding: utf-8 -*-
 # Pitivi video editor
-#
-#       pitivi/dialogs/clipmediaprops.py
-#
 # Copyright (c) 2011, Parthasarathi Susarla <partha@collabora.co.uk>
 #
 # This program is free software; you can redistribute it and/or
@@ -16,27 +14,29 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, see <http://www.gnu.org/licenses/>.
-
 import os
+from gettext import gettext as _
 
 from gi.repository import Gdk
 from gi.repository import Gst
 from gi.repository import Gtk
 
-from gettext import gettext as _
-
 from pitivi.configure import get_ui_dir
-from pitivi.utils.ui import frame_rates, audio_rates, \
-    audio_channels, pixel_aspect_ratios, get_value_from_model
+from pitivi.utils.ui import audio_channels
+from pitivi.utils.ui import audio_rates
+from pitivi.utils.ui import frame_rates
+from pitivi.utils.ui import get_value_from_model
+from pitivi.utils.ui import pixel_aspect_ratios
 
 
 class ClipMediaPropsDialog(object):
+    """Displays the properties of an asset.
 
-    """
-    Displays the properties of an asset, and allows applying them to a project.
+    Allows applying them to the project.
 
-    @type project: L{Project}
-    @type asset: L{GES.UriClipAsset}
+    Attributes:
+        project (Project): The project.
+        asset (GES.UriClipAsset): The displayed asset.
     """
 
     def __init__(self, project, asset):
@@ -74,7 +74,7 @@ class ClipMediaPropsDialog(object):
         self.video_header_label = builder.get_object("label2")
 
     def run(self):
-        """Set up widgets and run the dialog"""
+        """Sets up widgets and run the dialog."""
         # TODO: in "onApplyButtonClicked", we only use the first stream...
         # If we have multiple audio or video streams, we should reflect that
         # in the UI, instead of acting as if there was only one. But that means
@@ -109,7 +109,7 @@ class ClipMediaPropsDialog(object):
                 else:
                     foo = str(framerate_num) + "/" + str(framerate_denom)
                     # Translators: a label showing an invalid framerate value
-                    self.frame_rate.set_text(_("invalid (%s fps)" % foo))
+                    self.frame_rate.set_text(_("invalid (%s fps)") % foo)
                     self.framerate_checkbutton.set_active(False)
                     # For consistency, insensitize the checkbox AND value
                     # labels
@@ -135,10 +135,11 @@ class ClipMediaPropsDialog(object):
             self.video_header_label.set_markup("<b>" + _("Image:") + "</b>")
 
         self.dialog.connect("key-press-event", self._keyPressCb)
+        self.dialog.connect("response", self.__response_cb)
         self.dialog.run()
 
-    def _applyButtonCb(self, unused_button):
-        """Apply widget values to the project"""
+    def _apply(self):
+        """Applies the widgets values to the project."""
         project = self.project
         if self.has_video:
             # This also handles the case where the video is a still image
@@ -158,10 +159,10 @@ class ClipMediaPropsDialog(object):
                 project.audiochannels = audio.get_channels()
             if self.samplerate_checkbutton.get_active():
                 project.audiorate = audio.get_sample_rate()
-        self.dialog.destroy()
 
-    def _cancelButtonCb(self, unused_button):
-        """Destroy the dialog"""
+    def __response_cb(self, unused_dialog, response_id):
+        if response_id == 1:
+            self._apply()
         self.dialog.destroy()
 
     def _keyPressCb(self, unused_widget, event):
