@@ -1,7 +1,5 @@
+# -*- coding: utf-8 -*-
 # Pitivi video editor
-#
-#       pitivi/tabsmanager.py
-#
 # Copyright (c) 2005, Edward Hervey <bilboed@bilboed.com>
 #
 # This program is free software; you can redistribute it and/or
@@ -18,40 +16,50 @@
 # License along with this program; if not, write to the
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
-
-from gi.repository import Gtk
+"""Gtk.Notebook helpers."""
 from gi.repository import Gdk
-from pitivi.utils.ui import SPACING
+from gi.repository import Gtk
+
+from pitivi.settings import ConfigError
 from pitivi.settings import GlobalSettings
+from pitivi.utils.loggable import Loggable
+from pitivi.utils.ui import SPACING
 
 
-class BaseTabs(Gtk.Notebook):
+class BaseTabs(Gtk.Notebook, Loggable):
+    """Notebook which can detach its tabs to new windows.
 
-    """
-    @type app: Pitivi
+    Persists which of its tabs are detached.
+
+    Attributes:
+        app (Pitivi): The app.
     """
 
     def __init__(self, app):
         Gtk.Notebook.__init__(self)
+        Loggable.__init__(self)
         self.set_border_width(SPACING)
         self.set_scrollable(True)
-        self.connect("create-window", self._createWindowCb)
-        self.settings = app.settings  # To save/restore states of detached tabs
+        self.settings = app.settings
         notebook_widget_settings = self.get_settings()
         notebook_widget_settings.props.gtk_dnd_drag_threshold = 1
 
+    # pylint: disable=arguments-differ
     def append_page(self, child, label):
         child_name = label.get_text()
         Gtk.Notebook.append_page(self, child, label)
         self._set_child_properties(child, label)
         label.show()
 
-        self._createDefaultConfig(child_name)
+        self._create_default_config(child_name)
         docked = getattr(self.settings, child_name + "docked")
         if docked is False:
-            self.createWindow(child, created_by_signal=False)
-        # Wait till the tab or floating window is ready before showing
-        # contents:
+            # Restore a previously undocked state.
+            notebook = self._create_window(child)
+            original_position = self.page_num(child)
+            self.remove_page(original_position)
+            # Add the tab to the notebook in our newly created window.
+            notebook.append_page(child, Gtk.Label(label=child_name))
         child.show()
 
     def _set_child_properties(self, child, label):
@@ -60,8 +68,8 @@ class BaseTabs(Gtk.Notebook):
         self.child_set_property(child, "tab-fill", True)
         label.props.xalign = 0.0
 
-    def _detachedComponentWindowDestroyCb(self, window, child,
-                                          original_position, child_name):
+    def __detached_window_destroyed_cb(self, window, child,
+                                       original_position, child_name):
         notebook = window.get_child()
         position = notebook.page_num(child)
         notebook.remove_page(position)
@@ -70,22 +78,19 @@ class BaseTabs(Gtk.Notebook):
         self.insert_page(child, label, original_position)
         self._set_child_properties(child, label)
 
-    def _createWindowCb(self, unused_from_notebook, child, unused_x, unused_y):
-        """
-        Callback that occurs when tearing off a tab to create a new window
-        """
-        # from_notebook == BaseTabs instance == self. It is a group of tabs.
-        # child is the widget inside the notebook's tab's content area.
-        # The return statement here is important to provide the notebook widget
-        # that gtk should insert into the window at the end:
-        return self.createWindow(child)
+    def do_create_window(self, child, unused_x, unused_y):
+        """Handles the detachment of a page.
 
-    def createWindow(self, child, created_by_signal=True):
-        """
-        Create a window out of the tab. This can be called by _createWindowCb
-        or manually (to restore a previously undocked state) by specifying
-        created_by_signal=False.
+        Args:
+            child (Gtk.Widget): The detached page.
+
+        Returns:
+            Gtk.Notebook: The notebook the page should be attached to.
         """
+        return self._create_window(child)
+
+    def _create_window(self, child):
+        """Creates a separate window for the specified child."""
         original_position = self.page_num(child)
         child_name = self.get_tab_label(child).get_text()
         window = Gtk.Window()
@@ -108,27 +113,16 @@ class BaseTabs(Gtk.Notebook):
         window.show_all()
         window.move(x, y)
         window.connect(
-            "configure-event", self._detachedComponentWindowConfiguredCb,
+            "configure-event", self.__detached_window_configure_cb,
             child_name)
         window.connect(
-            "destroy", self._detachedComponentWindowDestroyCb, child,
+            "destroy", self.__detached_window_destroyed_cb, child,
             original_position, child_name)
 
-        if not created_by_signal:
-            # Delete the tab from the original notebook (since it was not
-            # torn off by GTK) and add its contents to our newly created window
-            self.remove_page(original_position)
-            notebook.append_page(child, Gtk.Label(label=child_name))
-        else:
-            return notebook
+        return notebook
 
-    def _detachedComponentWindowConfiguredCb(self, window, event, child_name):
-        """
-        When the user configures the detached window
-        (changes its size, position, etc.), save the settings.
-
-        The config key's name depends on the name (label) of the tab widget.
-        """
+    def __detached_window_configure_cb(self, window, event, child_name):
+        """Saves the position and size of the specified window."""
         # get_position() takes the window manager's decorations into account
         position = window.get_position()
         setattr(self.settings, child_name + "width", event.width)
@@ -136,12 +130,14 @@ class BaseTabs(Gtk.Notebook):
         setattr(self.settings, child_name + "x", position[0])
         setattr(self.settings, child_name + "y", position[1])
 
-    def _createDefaultConfig(self, child_name):
-        """
-        If they do not exist already, create default settings
-        to save the state of a detachable widget.
-        """
-        GlobalSettings.addConfigSection(child_name)
+    def _create_default_config(self, child_name):
+        """Creates default settings to save the state of a detachable widget."""
+        try:
+            GlobalSettings.addConfigSection(child_name)
+        except ConfigError:
+            self.info("Section %s already exists", child_name)
+            return
+
         GlobalSettings.addConfigOption(child_name + "docked",
                                        section=child_name,
                                        key="docked",
