@@ -19,13 +19,13 @@
 from gi.repository import GES
 from gi.repository import GObject
 from gi.repository import Gst
+from gi.repository import GstController
 
 from pitivi.effects import PROPS_TO_IGNORE
-from pitivi.undo.undo import ExpandableUndoableAction
+from pitivi.undo.undo import Action
 from pitivi.undo.undo import FinalizingAction
 from pitivi.undo.undo import GObjectObserver
 from pitivi.undo.undo import MetaContainerObserver
-from pitivi.undo.undo import SimpleUndoableAction
 from pitivi.undo.undo import UndoableAction
 from pitivi.undo.undo import UndoableAutomaticObjectAction
 from pitivi.utils.loggable import Loggable
@@ -55,6 +55,12 @@ class TrackElementPropertyChanged(UndoableAction):
         self.old_value = old_value
         self.new_value = new_value
 
+    def __repr__(self):
+        return "<TrackElementPropertyChanged %s.%s: %s -> %s>" % (self.track_element,
+                                                                  self.property_name,
+                                                                  self.old_value,
+                                                                  self.new_value)
+
     def do(self):
         self.track_element.set_child_property(
             self.property_name, self.new_value)
@@ -72,7 +78,6 @@ class TrackElementPropertyChanged(UndoableAction):
                 isinstance(self.new_value, GObject.GEnum):
             value = int(self.new_value)
         st['value'] = value
-
         return st
 
 
@@ -118,12 +123,34 @@ class TimelineElementObserver(Loggable):
         old_value = self._properties[prop_name]
         res, new_value = ges_timeline_element.get_child_property(prop_name)
         assert res, prop_name
+        if old_value == new_value:
+            # Nothing to see here.
+            return
+
         action = TrackElementPropertyChanged(
             ges_timeline_element, prop_name, old_value, new_value)
         self._properties[prop_name] = new_value
         self.action_log.push(action)
 
 
+class TrackElementObserver(TimelineElementObserver):
+    """Monitors the props of a track element.
+
+    Reports UndoableActions.
+
+    Args:
+        ges_track_element (GES.TrackElement): The object to be monitored.
+    """
+
+    def __init__(self, ges_track_element, action_log):
+        TimelineElementObserver.__init__(self, ges_track_element, action_log)
+        self.gobject_observer = GObjectObserver(ges_track_element, ("active",), action_log)
+
+    def release(self):
+        TimelineElementObserver.release(self)
+        self.gobject_observer.release()
+
+
 class TrackElementAction(UndoableAction):
 
     def __init__(self, clip, track_element):
@@ -149,7 +176,10 @@ class TrackElementAction(UndoableAction):
         self.clip.remove(self.track_element)
 
 
-class EffectAddedAction(TrackElementAction):
+class TrackElementAdded(TrackElementAction):
+
+    def __repr__(self):
+        return "<TrackElementAdded %s, %s>" % (self.clip, self.track_element)
 
     def do(self):
         self.add()
@@ -161,12 +191,16 @@ class EffectAddedAction(TrackElementAction):
         st = Gst.Structure.new_empty("container-add-child")
         st["container-name"] = self.clip.get_name()
         st["asset-id"] = self.track_element.get_id()
-        st["child-type"] = GObject.type_name(
-            self.track_element.get_asset().get_extractable_type())
+        asset = self.track_element.get_asset()
+        if asset:
+            st["child-type"] = GObject.type_name(asset.get_extractable_type())
         return st
 
 
-class EffectRemovedAction(TrackElementAction):
+class TrackElementRemoved(TrackElementAction):
+
+    def __repr__(self):
+        return "<TrackElementRemoved %s, %s>" % (self.clip, self.track_element)
 
     def do(self):
         self.remove()
@@ -233,22 +267,42 @@ class ControlSourceObserver(GObject.Object):
         self.action_log.push(action)
 
 
-class ClipAdded(UndoableAction):
+class ClipAction(UndoableAction):
 
     def __init__(self, layer, clip):
         UndoableAction.__init__(self)
         self.layer = layer
         self.clip = clip
 
-    def do(self):
+    def add(self):
         self.clip.set_name(None)
+        children = self.clip.get_children(False)
         self.layer.add_clip(self.clip)
+        # GES adds children if the clip had none. Make sure they are removed.
+        for child in self.clip.get_children(False):
+            if child not in children:
+                self.clip.remove(child)
         self.layer.get_timeline().get_asset().pipeline.commit_timeline()
 
-    def undo(self):
+    def _child_added_cb(self, clip, track_element):
+        clip.remove(track_element)
+
+    def remove(self):
         self.layer.remove_clip(self.clip)
         self.layer.get_timeline().get_asset().pipeline.commit_timeline()
 
+
+class ClipAdded(ClipAction):
+
+    def __repr__(self):
+        return "<ClipAdded %s>" % self.clip
+
+    def do(self):
+        self.add()
+
+    def undo(self):
+        self.remove()
+
     def asScenarioAction(self):
         timeline = self.layer.get_timeline()
         if hasattr(self.layer, "splitting_object") and \
@@ -268,14 +322,15 @@ class ClipAdded(UndoableAction):
         return st
 
 
-class ClipRemoved(ExpandableUndoableAction):
+class ClipRemoved(ClipAction):
 
     def __init__(self, layer, clip):
-        ExpandableUndoableAction.__init__(self)
-        self.layer = layer
-        self.clip = clip
+        ClipAction.__init__(self, layer, clip)
         self.transition_removed_actions = []
 
+    def __repr__(self):
+        return "<ClipRemoved %s>" % self.clip
+
     def expand(self, action):
         if not isinstance(action, TransitionClipRemovedAction):
             return False
@@ -283,13 +338,10 @@ class ClipRemoved(ExpandableUndoableAction):
         return True
 
     def do(self):
-        self.layer.remove_clip(self.clip)
-        self.layer.get_timeline().get_asset().pipeline.commit_timeline()
+        self.remove()
 
     def undo(self):
-        self.clip.set_name(None)
-        self.layer.add_clip(self.clip)
-        self.layer.get_timeline().get_asset().pipeline.commit_timeline()
+        self.add()
         # Update the automatically created transitions.
         for action in self.transition_removed_actions:
             action.undo()
@@ -427,6 +479,9 @@ class LayerRemoved(UndoableAction):
         self.ges_timeline = ges_timeline
         self.ges_layer = ges_layer
 
+    def __repr__(self):
+        return "<LayerRemoved %s>" % self.ges_layer
+
     def do(self):
         self.ges_timeline.remove_layer(self.ges_layer)
         self.ges_timeline.get_asset().pipeline.commit_timeline()
@@ -448,6 +503,11 @@ class LayerMoved(UndoableAction):
         self.old_priority = old_priority
         self.priority = priority
 
+    def __repr__(self):
+        return "<LayerMoved %s: %s -> %s>" % (self.ges_layer,
+                                              self.old_priority,
+                                              self.priority)
+
     def do(self):
         self.ges_layer.props.priority = self.priority
 
@@ -525,38 +585,54 @@ class KeyframeChangedAction(UndoableAction):
         self.control_source.set(time, value)
 
 
-class ActivePropertyChanged(UndoableAction):
+class ControlSourceSetAction(UndoableAction):
 
-    def __init__(self, effect_action, active):
+    def __init__(self, track_element, binding):
         UndoableAction.__init__(self)
-        self.effect_action = effect_action
-        self.active = not active
+        self.track_element = track_element
+        self.control_source = binding.props.control_source
+        self.property_name = binding.props.name
+        self.binding_type = "direct-absolute" if binding.props.absolute else "direct"
 
     def do(self):
-        self.effect_action.track_element.active = self.active
-        self.active = not self.active
+        self.track_element.set_control_source(self.control_source,
+                                              self.property_name, self.binding_type)
 
     def undo(self):
-        self.effect_action.track_element.active = self.active
-        self.active = not self.active
-
-
-class ControlSourceSetAction(SimpleUndoableAction):
-
-    def __init__(self, action_info):
-        SimpleUndoableAction.__init__(self)
-        self.action_info = action_info
+        assert self.track_element.remove_control_binding(self.property_name)
 
     def asScenarioAction(self):
         st = Gst.Structure.new_empty("set-control-source")
-        for key, value in self.action_info.items():
-            st.set_value(key, value)
-        st.set_value("binding-type", "direct")
+        st.set_value("element-name", self.track_element.get_name())
+        st.set_value("property-name", self.property_name)
+        st.set_value("binding-type", self.binding_type)
         st.set_value("source-type", "interpolation")
         st.set_value("interpolation-mode", "linear")
         return st
 
 
+class ControlSourceRemoveAction(UndoableAction):
+
+    def __init__(self, track_element, binding):
+        UndoableAction.__init__(self)
+        self.track_element = track_element
+        self.control_source = binding.props.control_source
+        self.property_name = binding.props.name
+        self.binding_type = "direct-absolute" if binding.props.absolute else "direct"
+
+    def do(self):
+        assert self.track_element.remove_control_binding(self.property_name)
+
+    def undo(self):
+        self.track_element.set_control_source(self.control_source,
+                                              self.property_name, self.binding_type)
+
+    def asScenarioAction(self):
+        st = Gst.Structure.new_empty("remove-control-source")
+        st.set_value("element-name", self.track_element.get_name())
+        st.set_value("property-name", self.property_name)
+        return st
+
 class LayerObserver(MetaContainerObserver, Loggable):
     """Monitors a Layer and reports UndoableActions.
 
@@ -611,11 +687,14 @@ class LayerObserver(MetaContainerObserver, Loggable):
         clip_observer = self.clip_observers.pop(ges_clip)
         clip_observer.release()
 
-    def _controlBindingAddedCb(self, track_element, binding):
+    def _control_binding_added_cb(self, track_element, binding):
         self._connectToControlSource(track_element, binding)
-        action_info = {"element-name": track_element.get_name(),
-                       "property-name": binding.props.name}
-        action = ControlSourceSetAction(action_info)
+        action = ControlSourceSetAction(track_element, binding)
+        self.action_log.push(action)
+
+    def _control_binding_removed_cb(self, track_element, binding):
+        self._disconnectFromControlSource(binding)
+        action = ControlSourceRemoveAction(track_element, binding)
         self.action_log.push(action)
 
     def _connectToTrackElement(self, track_element):
@@ -634,13 +713,18 @@ class LayerObserver(MetaContainerObserver, Loggable):
         for prop, binding in track_element.get_all_control_bindings().items():
             self._connectToControlSource(track_element, binding)
         track_element.connect("control-binding-added",
-                              self._controlBindingAddedCb)
+                              self._control_binding_added_cb)
+        track_element.connect("control-binding-removed",
+                              self._control_binding_removed_cb)
         if isinstance(track_element, GES.BaseEffect) or \
                 isinstance(track_element, GES.VideoSource):
-            observer = TimelineElementObserver(track_element, self.action_log)
+            observer = TrackElementObserver(track_element, self.action_log)
             self.track_element_observers[track_element] = observer
 
     def _disconnectFromTrackElement(self, track_element):
+        if not isinstance(track_element, GES.VideoTransition):
+            track_element.disconnect_by_func(self._control_binding_added_cb)
+            track_element.disconnect_by_func(self._control_binding_removed_cb)
         for prop, binding in track_element.get_all_control_bindings().items():
             self._disconnectFromControlSource(binding)
         observer = self.track_element_observers.pop(track_element, None)
@@ -652,9 +736,10 @@ class LayerObserver(MetaContainerObserver, Loggable):
         control_source = binding.props.control_source
         action_info = {"element-name": track_element.get_name(),
                        "property-name": binding.props.name}
-        observer = ControlSourceObserver(control_source, self.action_log,
-                                         action_info)
-        self.keyframe_observers[control_source] = observer
+        if control_source not in self.keyframe_observers:
+            observer = ControlSourceObserver(control_source, self.action_log,
+                                             action_info)
+            self.keyframe_observers[control_source] = observer
 
     def _disconnectFromControlSource(self, binding):
         control_source = binding.props.control_source
@@ -680,20 +765,13 @@ class LayerObserver(MetaContainerObserver, Loggable):
 
     def _clipTrackElementAddedCb(self, clip, ges_track_element):
         self._connectToTrackElement(ges_track_element)
-        if isinstance(ges_track_element, GES.BaseEffect):
-            action = EffectAddedAction(clip, ges_track_element)
-            self.action_log.push(action)
+        action = TrackElementAdded(clip, ges_track_element)
+        self.action_log.push(action)
 
     def _clipTrackElementRemovedCb(self, clip, ges_track_element):
         self.debug("%s REMOVED from %s", ges_track_element, clip)
         self._disconnectFromTrackElement(ges_track_element)
-        if isinstance(ges_track_element, GES.BaseEffect):
-            action = EffectRemovedAction(clip, ges_track_element)
-            self.action_log.push(action)
-
-    def _trackElementActiveChangedCb(self, track_element, active, add_effect_action):
-        """Handles an effect is (de)activated on a clip in the timeline."""
-        action = ActivePropertyChanged(add_effect_action, active)
+        action = TrackElementRemoved(clip, ges_track_element)
         self.action_log.push(action)
 
     def __layer_moved_cb(self, ges_layer, unused_param):
@@ -710,6 +788,9 @@ class TimelineElementAddedToGroup(UndoableAction):
         self.ges_group = ges_group
         self.ges_timeline_element = ges_timeline_element
 
+    def __repr__(self):
+        return "<TimelineElementAddedToGroup %s, %s>" % (self.ges_group, self.ges_timeline_element)
+
     def do(self):
         self.ges_group.add(self.ges_timeline_element)
 
@@ -724,6 +805,9 @@ class TimelineElementRemovedFromGroup(UndoableAction):
         self.ges_group = ges_group
         self.ges_timeline_element = ges_timeline_element
 
+    def __repr__(self):
+        return "<TimelineElementRemovedFromGroup %s, %s>" % (self.ges_group, self.ges_timeline_element)
+
     def do(self):
         self.ges_group.remove(self.ges_timeline_element)
 
@@ -802,7 +886,7 @@ class TimelineObserver(Loggable):
 
     def _connect_to_group(self, ges_group):
         if not ges_group.props.serialize:
-            return
+            return False
 
         # A group is added when it gets its first element, thus
         # when undoing/redoing a group can be added multiple times.
@@ -811,9 +895,12 @@ class TimelineObserver(Loggable):
         if ges_group not in self.group_observers:
             group_observer = GroupObserver(ges_group, self.action_log)
             self.group_observers[ges_group] = group_observer
+        return True
 
     def __group_added_cb(self, unused_ges_timeline, ges_group):
-        self._connect_to_group(ges_group)
+        if not self._connect_to_group(ges_group):
+            return
+
         # This should be a single clip.
         for ges_clip in ges_group.get_children(recursive=False):
             action = TimelineElementAddedToGroup(ges_group, ges_clip)
