@@ -82,23 +82,25 @@ def call_false(function, *args, **kwargs):
     return False
 
 
-def get_proxy_target(obj):
-    if isinstance(obj, GES.UriClip):
-        asset = obj.get_asset()
-    elif isinstance(obj, GES.TrackElement):
-        asset = obj.get_parent().get_asset()
-    else:
-        asset = obj
-
-    target = asset.get_proxy_target()
-    if target and target.get_error() is None:
-        asset = target
-
-    return asset
-
-
 # ------------------------------ URI helpers --------------------------------
 
+def is_valid_file(path):
+    """Returns whether a path is or can be a regular file."""
+    if os.path.isfile(path):
+        return True
+    if os.path.exists(path):
+        # The path is not a regular file
+        return False
+
+    try:
+        # The path doesn't exist, so open(path, "w") is safe to use.
+        open(path, "w").close()
+        os.unlink(path)
+        return True
+    except IOError:
+        return False
+
+
 def isWritable(path):
     """Returns whether the file/path is writable."""
     try:
@@ -142,9 +144,14 @@ def filename_from_uri(uri):
 
     Excludes the path to the file.
 
-    Can be used in UI elements or to shorten debug statements.
+    Can be used in UI elements.
+
+    Returns:
+        str: The markup escaped filename
     """
-    return os.path.basename(path_from_uri(uri))
+    base_name = os.path.basename(path_from_uri(uri))
+    safe_base_name = GLib.markup_escape_text(base_name)
+    return safe_base_name
 
 
 def quote_uri(uri):
@@ -164,27 +171,43 @@ def quote_uri(uri):
 class PathWalker(Thread):
     """Thread for recursively searching in a list of directories."""
 
-    def __init__(self, paths, callback):
+    def __init__(self, uris, callback):
         Thread.__init__(self)
-        self.log("New PathWalker for %s", paths)
-        self.paths = paths
+        self.log("New PathWalker for %s", uris)
+        self.uris = uris
         self.callback = callback
         self.stopme = threading.Event()
 
+    def _scan(self, uris):
+        """Scans the URIs and yields the file URIs."""
+        for uri in uris:
+            if self.stopme.is_set():
+                return
+            url = urlparse(uri)
+            if not url.scheme == 'file':
+                self.fixme("Unsupported URI: %s", uri)
+                continue
+            path = unquote(url.path)
+            if os.path.isfile(path):
+                yield uri
+            elif os.path.isdir(path):
+                yield from self._scan_dir(path)
+            else:
+                self.warning("Unusable, not a file nor a dir: %s, %s", uri, path)
+
+    def _scan_dir(self, folder):
+        """Scans the folder recursively and yields the URIs of the files."""
+        self.log("Scanning folder %s", folder)
+        for path, dirs, files in os.walk(folder):
+            if self.stopme.is_set():
+                return
+            for afile in files:
+                yield Gst.filename_to_uri(os.path.join(path, afile))
+
     def process(self):
-        for folder in self.paths:
-            self.log("folder %s" % folder)
-            if folder.startswith("file://"):
-                folder = unquote(folder[len("file://"):])
-            for path, dirs, files in os.walk(folder):
-                if self.stopme.isSet():
-                    return
-                uris = []
-                for afile in files:
-                    uris.append(quote_uri("file://%s" %
-                                          os.path.join(path, afile)))
-                if uris:
-                    GLib.idle_add(self.callback, uris)
+        uris = list(self._scan(self.uris))
+        if uris:
+            GLib.idle_add(self.callback, uris)
 
     def abort(self):
         self.stopme.set()
@@ -285,3 +308,90 @@ def unicode_error_dialog():
     dialog.set_title(_("Error while decoding a string"))
     dialog.run()
     dialog.destroy()
+
+
+def intersect(v1, v2):
+    s = Gst.Structure('t', t=v1).intersect(Gst.Structure('t', t=v2))
+    if s:
+        return s['t']
+
+    return None
+
+
+def fixate_caps_with_default_values(template, restrictions, default_values,
+                                    prev_vals=None):
+    """Fixates @template taking into account other restriction values.
+
+    The resulting caps will only contain the fields from @default_values,
+    @restrictions and @prev_vals
+
+    Args:
+        template (Gst.Caps) : The pad template to fixate.
+        restrictions (Gst.Caps): Restriction caps to be used to fixate
+            @template. This is the minimum requested
+            restriction. Can be None
+        default_values (dict) : Dictionary containing the minimal fields
+            to be fixated and some default values (can be ranges).
+        prev_vals (Optional[Gst.Caps]) : Some values that were previously
+            used, and should be kept instead of the default values if possible.
+
+    Returns:
+        Gst.Caps: The caps resulting from the previously defined operations.
+    """
+    res = Gst.Caps.new_empty()
+    fields = set(default_values.keys())
+    if restrictions:
+        for struct in restrictions:
+            fields.update(struct.keys())
+
+        log.log("utils", "Intersect template %s with the restriction %s",
+                template, restrictions)
+        tmp = template.intersect(restrictions)
+
+        if not tmp:
+            log.warning("utils",
+                        "No common format between template %s and restrictions %s",
+                        template, restrictions)
+        else:
+            template = tmp
+
+    for struct in template:
+        struct = struct.copy()
+        for field in fields:
+            prev_val = None
+            default_val = default_values.get(field)
+            if prev_vals and prev_vals[0].has_field(field):
+                prev_val = prev_vals[0][field]
+
+            if not struct.has_field(field):
+                if prev_val:
+                    struct[field] = prev_val
+                elif default_val:
+                    struct[field] = default_val
+            else:
+                v = None
+                struct_val = struct[field]
+                if prev_val:
+                    v = intersect(struct_val, prev_val)
+                    if v is not None:
+                        struct[field] = v
+                if v is None and default_val:
+                    v = intersect(struct_val, default_val)
+                    if v is not None:
+                        struct[field] = v
+                else:
+                    log.debug("utils", "Field %s from %s is plainly fixated",
+                              field, struct)
+
+        struct = struct.copy()
+        for key in struct.keys():
+            if key not in fields:
+                struct.remove_field(key)
+
+        log.debug("utils", "Adding %s to resulting caps", struct)
+        res.append_structure(struct)
+
+    res.mini_object.refcount += 1
+    res = res.fixate()
+    log.debug("utils", "Fixated %s", res)
+    return res
