@@ -21,7 +21,8 @@
 
 /**
  * SECTION:gsttranscoder
- * @short_description: GStreamer Transcoder API
+ * @short_description: High level API to transcode media files
+ * from one format to any other format using the GStreamer framework.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,8 +38,9 @@ GST_DEBUG_CATEGORY_STATIC (gst_transcoder_debug);
 #define DEFAULT_POSITION GST_CLOCK_TIME_NONE
 #define DEFAULT_DURATION GST_CLOCK_TIME_NONE
 #define DEFAULT_POSITION_UPDATE_INTERVAL_MS 100
+#define DEFAULT_AVOID_REENCODING   FALSE
 
-static GQuark
+GQuark
 gst_transcoder_error_quark (void)
 {
   static GQuark quark;
@@ -60,6 +62,7 @@ enum
   PROP_DURATION,
   PROP_PIPELINE,
   PROP_POSITION_UPDATE_INTERVAL,
+  PROP_AVOID_REENCODING,
   PROP_LAST
 };
 
@@ -67,7 +70,6 @@ enum
 {
   SIGNAL_POSITION_UPDATED,
   SIGNAL_DURATION_CHANGED,
-  SIGNAL_BUFFERING,
   SIGNAL_DONE,
   SIGNAL_ERROR,
   SIGNAL_WARNING,
@@ -225,6 +227,11 @@ gst_transcoder_class_init (GstTranscoderClass * klass)
       0, 10000, DEFAULT_POSITION_UPDATE_INTERVAL_MS,
       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  param_specs[PROP_AVOID_REENCODING] =
+      g_param_spec_boolean ("avoid-reencoding", "Avoid re-encoding",
+      "Whether to re-encode portions of compatible video streams that lay on segment boundaries",
+      DEFAULT_AVOID_REENCODING, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (gobject_class, PROP_LAST, param_specs);
 
   signals[SIGNAL_POSITION_UPDATED] =
@@ -245,12 +252,12 @@ gst_transcoder_class_init (GstTranscoderClass * klass)
   signals[SIGNAL_ERROR] =
       g_signal_new ("error", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS, 0, NULL,
-      NULL, NULL, G_TYPE_NONE, 1, G_TYPE_ERROR);
+      NULL, NULL, G_TYPE_NONE, 2, G_TYPE_ERROR, GST_TYPE_STRUCTURE);
 
   signals[SIGNAL_WARNING] =
       g_signal_new ("warning", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS, 0, NULL,
-      NULL, NULL, G_TYPE_NONE, 1, G_TYPE_ERROR);
+      NULL, NULL, G_TYPE_NONE, 2, G_TYPE_ERROR, GST_TYPE_STRUCTURE);
 }
 
 static void
@@ -356,6 +363,10 @@ gst_transcoder_set_property (GObject * object, guint prop_id,
       self->profile = g_value_dup_object (value);
       GST_OBJECT_UNLOCK (self);
       break;
+    case PROP_AVOID_REENCODING:
+      g_object_set (self->transcodebin, "avoid-reencoding",
+          g_value_get_boolean (value), NULL);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -416,6 +427,15 @@ gst_transcoder_get_property (GObject * object, guint prop_id,
       g_value_set_object (value, self->profile);
       GST_OBJECT_UNLOCK (self);
       break;
+    case PROP_AVOID_REENCODING:
+    {
+      gboolean avoid_reencoding;
+
+      g_object_get (self->transcodebin, "avoid-reencoding", &avoid_reencoding,
+          NULL);
+      g_value_set_boolean (value, avoid_reencoding);
+      break;
+    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -476,7 +496,7 @@ tick_cb (gpointer user_data)
 
     if (g_signal_handler_find (self, G_SIGNAL_MATCH_ID,
             signals[SIGNAL_POSITION_UPDATED], 0, NULL, NULL, NULL) != 0) {
-      PositionUpdatedSignalData *data = g_new (PositionUpdatedSignalData, 1);
+      PositionUpdatedSignalData *data = g_new0 (PositionUpdatedSignalData, 1);
 
       data->transcoder = g_object_ref (self);
       data->position = position;
@@ -518,35 +538,41 @@ typedef struct
 {
   GstTranscoder *transcoder;
   GError *err;
-} ErrorSignalData;
+  GstStructure *details;
+} IssueSignalData;
 
 static void
 error_dispatch (gpointer user_data)
 {
-  ErrorSignalData *data = user_data;
+  IssueSignalData *data = user_data;
 
-  g_signal_emit (data->transcoder, signals[SIGNAL_ERROR], 0, data->err);
+  g_signal_emit (data->transcoder, signals[SIGNAL_ERROR], 0, data->err,
+      data->details);
 }
 
 static void
-free_error_signal_data (ErrorSignalData * data)
+free_issue_signal_data (IssueSignalData * data)
 {
   g_object_unref (data->transcoder);
+  if (data->details)
+    gst_structure_free (data->details);
   g_clear_error (&data->err);
   g_free (data);
 }
 
 static void
-emit_error (GstTranscoder * self, GError * err)
+emit_error (GstTranscoder * self, GError * err, const GstStructure * details)
 {
   if (g_signal_handler_find (self, G_SIGNAL_MATCH_ID,
           signals[SIGNAL_ERROR], 0, NULL, NULL, NULL) != 0) {
-    ErrorSignalData *data = g_new (ErrorSignalData, 1);
+    IssueSignalData *data = g_new0 (IssueSignalData, 1);
 
     data->transcoder = g_object_ref (self);
     data->err = g_error_copy (err);
+    if (details)
+      data->details = gst_structure_copy (details);
     gst_transcoder_signal_dispatcher_dispatch (self->signal_dispatcher, self,
-        error_dispatch, data, (GDestroyNotify) free_error_signal_data);
+        error_dispatch, data, (GDestroyNotify) free_issue_signal_data);
   }
 
   g_error_free (err);
@@ -568,47 +594,33 @@ dump_dot_file (GstTranscoder * self, const gchar * name)
   full_name = g_strdup_printf ("gst-transcoder.%p.%s", self, name);
 
   GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (self->transcodebin),
-      GST_DEBUG_GRAPH_SHOW_ALL, full_name);
+      GST_DEBUG_GRAPH_SHOW_VERBOSE, full_name);
 
   g_free (full_name);
 }
 
-typedef struct
-{
-  GstTranscoder *transcoder;
-  GError *err;
-} WarningSignalData;
-
 static void
 warning_dispatch (gpointer user_data)
 {
-  WarningSignalData *data = user_data;
+  IssueSignalData *data = user_data;
 
-  g_signal_emit (data->transcoder, signals[SIGNAL_WARNING], 0, data->err);
+  g_signal_emit (data->transcoder, signals[SIGNAL_WARNING], 0, data->err,
+      data->details);
 }
 
 static void
-free_warning_signal_data (WarningSignalData * data)
+emit_warning (GstTranscoder * self, GError * err, const GstStructure * details)
 {
-  g_object_unref (data->transcoder);
-  g_clear_error (&data->err);
-  g_free (data);
-}
-
-static void
-emit_warning (GstTranscoder * self, GError * err)
-{
-  GST_ERROR_OBJECT (self, "Warning: %s (%s, %d)", err->message,
-      g_quark_to_string (err->domain), err->code);
-
   if (g_signal_handler_find (self, G_SIGNAL_MATCH_ID,
           signals[SIGNAL_WARNING], 0, NULL, NULL, NULL) != 0) {
-    WarningSignalData *data = g_new (WarningSignalData, 1);
+    IssueSignalData *data = g_new0 (IssueSignalData, 1);
 
     data->transcoder = g_object_ref (self);
     data->err = g_error_copy (err);
+    if (details)
+      data->details = gst_structure_copy (details);
     gst_transcoder_signal_dispatcher_dispatch (self->signal_dispatcher, self,
-        warning_dispatch, data, (GDestroyNotify) free_warning_signal_data);
+        warning_dispatch, data, (GDestroyNotify) free_issue_signal_data);
   }
 
   g_error_free (err);
@@ -617,39 +629,33 @@ emit_warning (GstTranscoder * self, GError * err)
 static void
 error_cb (G_GNUC_UNUSED GstBus * bus, GstMessage * msg, gpointer user_data)
 {
+  GError *err;
   GstTranscoder *self = GST_TRANSCODER (user_data);
-  GError *err, *transcoder_err;
-  gchar *name, *debug, *message, *full_message;
+  gchar *name, *debug, *message;
+  GstStructure *details = NULL;
 
   dump_dot_file (self, "error");
 
   gst_message_parse_error (msg, &err, &debug);
+  gst_message_parse_error_details (msg, (const GstStructure **) &details);
+
+  if (!details)
+    details = gst_structure_new_empty ("details");
+  else
+    details = gst_structure_copy (details);
 
   name = gst_object_get_path_string (msg->src);
   message = gst_error_get_message (err->domain, err->code);
 
-  if (debug)
-    full_message =
-        g_strdup_printf ("Error from element %s: %s\n%s\n%s", name, message,
-        err->message, debug);
-  else
-    full_message =
-        g_strdup_printf ("Error from element %s: %s\n%s", name, message,
-        err->message);
-
-  GST_ERROR_OBJECT (self, "ERROR: from element %s: %s\n", name, err->message);
-  if (debug != NULL)
-    GST_ERROR_OBJECT (self, "Additional debug info:\n%s\n", debug);
-
-  transcoder_err =
-      g_error_new_literal (GST_TRANSCODER_ERROR, GST_TRANSCODER_ERROR_FAILED,
-      full_message);
-  emit_error (self, transcoder_err);
+  gst_structure_set (details, "debug", G_TYPE_STRING, debug,
+      "msg-source-element-name", G_TYPE_STRING, "name",
+      "msg-source-type", G_TYPE_GTYPE, G_OBJECT_TYPE (msg->src),
+      "msg-error", G_TYPE_STRING, message, NULL);
+  emit_error (self, g_error_copy (err), details);
 
   g_clear_error (&err);
   g_free (debug);
   g_free (name);
-  g_free (full_message);
   g_free (message);
 }
 
@@ -659,10 +665,12 @@ warning_cb (G_GNUC_UNUSED GstBus * bus, GstMessage * msg, gpointer user_data)
   GstTranscoder *self = GST_TRANSCODER (user_data);
   GError *err, *transcoder_err;
   gchar *name, *debug, *message, *full_message;
+  const GstStructure *details = NULL;
 
   dump_dot_file (self, "warning");
 
   gst_message_parse_warning (msg, &err, &debug);
+  gst_message_parse_warning_details (msg, &details);
 
   name = gst_object_get_path_string (msg->src);
   message = gst_error_get_message (err->domain, err->code);
@@ -684,7 +692,7 @@ warning_cb (G_GNUC_UNUSED GstBus * bus, GstMessage * msg, gpointer user_data)
   transcoder_err =
       g_error_new_literal (GST_TRANSCODER_ERROR, GST_TRANSCODER_ERROR_FAILED,
       full_message);
-  emit_warning (self, transcoder_err);
+  emit_warning (self, transcoder_err, details);
 
   g_clear_error (&err);
   g_free (debug);
@@ -735,7 +743,8 @@ clock_lost_cb (G_GNUC_UNUSED GstBus * bus, G_GNUC_UNUSED GstMessage * msg,
 
     if (state_ret == GST_STATE_CHANGE_FAILURE)
       emit_error (self, g_error_new (GST_TRANSCODER_ERROR,
-              GST_TRANSCODER_ERROR_FAILED, "Failed to handle clock loss"));
+              GST_TRANSCODER_ERROR_FAILED, "Failed to handle clock loss"),
+          NULL);
   }
 }
 
@@ -773,7 +782,7 @@ emit_duration_changed (GstTranscoder * self, GstClockTime duration)
 
   if (g_signal_handler_find (self, G_SIGNAL_MATCH_ID,
           signals[SIGNAL_DURATION_CHANGED], 0, NULL, NULL, NULL) != 0) {
-    DurationChangedSignalData *data = g_new (DurationChangedSignalData, 1);
+    DurationChangedSignalData *data = g_new0 (DurationChangedSignalData, 1);
 
     data->transcoder = g_object_ref (self);
     data->duration = duration;
@@ -858,7 +867,7 @@ request_state_cb (G_GNUC_UNUSED GstBus * bus, GstMessage * msg,
     emit_error (self, g_error_new (GST_TRANSCODER_ERROR,
             GST_TRANSCODER_ERROR_FAILED,
             "Failed to change to requested state %s",
-            gst_element_state_get_name (state)));
+            gst_element_state_get_name (state)), NULL);
 }
 
 static void
@@ -1007,20 +1016,40 @@ create_encoding_profile (const gchar * pname)
 
 /**
  * gst_transcoder_new:
+ * @source_uri: The URI of the media stream to transcode
+ * @dest_uri: The URI of the destination of the transcoded stream
+ * @encoding_profile: The serialized #GstEncodingProfile defining the output
+ * format. Have a look at the #GstEncodingProfile documentation to find more
+ * about the serialization format.
  *
  * Returns: a new #GstTranscoder instance
  */
 GstTranscoder *
 gst_transcoder_new (const gchar * source_uri,
-    const gchar * dest_uri, const gchar * encoding_target)
+    const gchar * dest_uri, const gchar * encoding_profile)
 {
   GstEncodingProfile *profile;
 
-  profile = create_encoding_profile (encoding_target);
+  profile = create_encoding_profile (encoding_profile);
+
+  if (!profile)
+    return NULL;
 
   return gst_transcoder_new_full (source_uri, dest_uri, profile, NULL);
 }
 
+/**
+ * gst_transcoder_new_full:
+ * @source_uri: The URI of the media stream to transcode
+ * @dest_uri: The URI of the destination of the transcoded stream
+ * @profile: The #GstEncodingProfile defining the output format
+ * have a look at the #GstEncodingProfile documentation to find more
+ * about the serialization format.
+ * @signal_dispatcher: The #GstTranscoderSignalDispatcher to be used
+ * to dispatch the various signals.
+ *
+ * Returns: a new #GstTranscoder instance
+ */
 GstTranscoder *
 gst_transcoder_new_full (const gchar * source_uri,
     const gchar * dest_uri, GstEncodingProfile * profile,
@@ -1050,11 +1079,12 @@ typedef struct
 } RunSyncData;
 
 static void
-_error_cb (GstTranscoder * self, GError * error, RunSyncData * data)
+_error_cb (GstTranscoder * self, GError * error, GstStructure * details,
+    RunSyncData * data)
 {
   g_mutex_lock (&data->m);
   data->done = TRUE;
-  if (data->user_error && *data->user_error)
+  if (data->user_error && (*data->user_error) == NULL)
     g_propagate_error (data->user_error, g_error_copy (error));
   g_cond_broadcast (&data->cond);
   g_mutex_unlock (&data->m);
@@ -1121,7 +1151,7 @@ gst_transcoder_run_async (GstTranscoder * self)
 
   if (state_ret == GST_STATE_CHANGE_FAILURE) {
     emit_error (self, g_error_new (GST_TRANSCODER_ERROR,
-            GST_TRANSCODER_ERROR_FAILED, "Could not start transcoding"));
+            GST_TRANSCODER_ERROR_FAILED, "Could not start transcoding"), NULL);
     return;
   } else if (state_ret == GST_STATE_CHANGE_NO_PREROLL) {
     self->is_live = TRUE;
@@ -1285,6 +1315,40 @@ gst_transcoder_get_pipeline (GstTranscoder * self)
   return val;
 }
 
+/**
+ * gst_transcoder_get_avoid_reencoding:
+ * @self: The #GstTranscoder to check whether reencoding is avoided or not.
+ *
+ * Returns: %TRUE if the transcoder tries to avoid reencoding streams where
+ * reencoding is not strictly needed, %FALSE otherwise.
+ */
+gboolean
+gst_transcoder_get_avoid_reencoding (GstTranscoder * self)
+{
+  gboolean val;
+
+  g_return_val_if_fail (GST_IS_TRANSCODER (self), FALSE);
+
+  g_object_get (self->transcodebin, "avoid-reencoding", &val, NULL);
+
+  return val;
+}
+
+/**
+ * gst_transcoder_set_avoid_reencoding:
+ * @self: The #GstTranscoder to set whether reencoding should be avoided or not.
+ * @avoid_reencoding: %TRUE if the transcoder should try to avoid reencoding
+ * streams where * reencoding is not strictly needed, %FALSE otherwise.
+ */
+void
+gst_transcoder_set_avoid_reencoding (GstTranscoder * self,
+    gboolean avoid_reencoding)
+{
+  g_return_if_fail (GST_IS_TRANSCODER (self));
+
+  g_object_set (self->transcodebin, "avoid-reencoding", avoid_reencoding, NULL);
+}
+
 #define C_ENUM(v) ((gint) v)
 #define C_FLAGS(v) ((guint) v)
 
@@ -1504,7 +1568,7 @@ static void
   GstTranscoderGMainContextSignalDispatcher *self =
       GST_TRANSCODER_G_MAIN_CONTEXT_SIGNAL_DISPATCHER (iface);
   GMainContextSignalDispatcherData *gsourcefunc_data =
-      g_new (GMainContextSignalDispatcherData, 1);
+      g_new0 (GMainContextSignalDispatcherData, 1);
 
   gsourcefunc_data->emitter = emitter;
   gsourcefunc_data->data = data;
