@@ -149,33 +149,52 @@ pad_added_cb (GstElement * decodebin, GstPad * pad, GstTranscodeBin * self)
 {
   GstCaps *caps;
   GstPad *sinkpad = NULL;
+  GstPadLinkReturn lret;
 
   caps = gst_pad_query_caps (pad, NULL);
 
   GST_DEBUG_OBJECT (decodebin, "Pad added, caps: %" GST_PTR_FORMAT, caps);
 
   g_signal_emit_by_name (self->encodebin, "request-pad", caps, &sinkpad);
-  if (caps)
-    gst_caps_unref (caps);
 
   if (sinkpad == NULL) {
-    GST_WARNING ("Couldn't get an encoding pad for pad %s:%s\n",
-        GST_DEBUG_PAD_NAME (pad));
+    gchar *stream_id = gst_pad_get_stream_id (pad);
+
+    GST_ELEMENT_WARNING_WITH_DETAILS (self, STREAM, FORMAT,
+        (NULL), ("Stream with caps: %" GST_PTR_FORMAT " can not be"
+            " encoded in the defined encoding formats",
+            caps),
+        ("can-t-encode-stream", G_TYPE_BOOLEAN, TRUE,
+            "stream-caps", GST_TYPE_CAPS, caps,
+            "stream-id", G_TYPE_STRING, stream_id, NULL));
+
+    g_free (stream_id);
     return;
   }
 
+  if (caps)
+    gst_caps_unref (caps);
+
   pad = _insert_filter (self, sinkpad, pad, caps);
-  if (G_UNLIKELY (gst_pad_link (pad, sinkpad) != GST_PAD_LINK_OK)) {
-    GstCaps *othercaps = gst_pad_get_current_caps (sinkpad);
+  lret = gst_pad_link (pad, sinkpad);
+  if (G_UNLIKELY (lret != GST_PAD_LINK_OK)) {
+    GstCaps *othercaps = gst_pad_query_caps (sinkpad, NULL);
     caps = gst_pad_get_current_caps (pad);
 
-    GST_ELEMENT_ERROR (self, CORE, PAD,
+    GST_ELEMENT_ERROR_WITH_DETAILS (self, CORE, PAD,
         (NULL),
-        ("Couldn't link pads \n\n%" GST_PTR_FORMAT "\n\n  and \n\n %"
-            GST_PTR_FORMAT "\n\n", caps, othercaps));
+        ("Couldn't link pads:\n    %" GST_PTR_FORMAT ": %" GST_PTR_FORMAT
+            "\nand:\n" "    %" GST_PTR_FORMAT ": %" GST_PTR_FORMAT "\n\n",
+            pad, caps, sinkpad, othercaps),
+        ("linking-error", GST_TYPE_PAD_LINK_RETURN, lret,
+            "source-pad", GST_TYPE_PAD, pad,
+            "source-caps", GST_TYPE_CAPS, caps,
+            "sink-pad", GST_TYPE_PAD, sinkpad,
+            "sink-caps", GST_TYPE_CAPS, othercaps, NULL));
 
     gst_caps_unref (caps);
-    gst_caps_unref (othercaps);
+    if (othercaps)
+      gst_caps_unref (othercaps);
   }
 
   gst_object_unref (sinkpad);
@@ -192,11 +211,10 @@ make_encodebin (GstTranscodeBin * self)
 
   self->encodebin = gst_element_factory_make ("encodebin", NULL);
   if (!self->encodebin)
-    goto no_decodebin;
-
-  g_object_set (self->encodebin, "profile", self->profile, NULL);
+    goto no_encodebin;
 
   gst_bin_add (GST_BIN (self), self->encodebin);
+  g_object_set (self->encodebin, "profile", self->profile, NULL);
 
   pad = gst_element_get_static_pad (self->encodebin, "src");
   if (!gst_ghost_pad_set_target (GST_GHOST_PAD_CAST (self->srcpad), pad)) {
@@ -212,7 +230,7 @@ make_encodebin (GstTranscodeBin * self)
   return gst_element_sync_state_with_parent (self->encodebin);
 
   /* ERRORS */
-no_decodebin:
+no_encodebin:
   {
     post_missing_plugin_error (GST_ELEMENT_CAST (self), "encodebin");
 
@@ -242,6 +260,32 @@ make_decodebin (GstTranscodeBin * self)
   if (!self->decodebin)
     goto no_decodebin;
 
+  if (self->avoid_reencoding) {
+    GstCaps *decodecaps;
+
+    g_object_get (self->decodebin, "caps", &decodecaps, NULL);
+    if (GST_IS_ENCODING_CONTAINER_PROFILE (self->profile)) {
+      GList *tmp;
+
+      decodecaps = gst_caps_make_writable (decodecaps);
+      for (tmp = (GList *)
+          gst_encoding_container_profile_get_profiles
+          (GST_ENCODING_CONTAINER_PROFILE (self->profile)); tmp;
+          tmp = tmp->next) {
+        GstCaps *encodecaps = gst_encoding_profile_get_format (tmp->data);
+        GstCaps *restrictions =
+            gst_encoding_profile_get_restriction (tmp->data);
+
+        if (!restrictions)
+          gst_caps_append (decodecaps, encodecaps);
+        else
+          gst_caps_unref (restrictions);
+      }
+    }
+    g_object_set (self->decodebin, "caps", decodecaps, NULL);
+    gst_caps_unref (decodecaps);
+  }
+
   g_signal_connect (self->decodebin, "pad-added", G_CALLBACK (pad_added_cb),
       self);
 
