@@ -32,18 +32,24 @@ THICK = LAYER_HEIGHT
 
 
 class BaseTestTimeline(common.TestCase):
+    """Test case with tools for setting up a timeline."""
 
-    def addClipsSimple(self, timeline, num_clips):
-        layer = timeline.ges_timeline.append_layer()
+    def add_clip(self, layer, start, inpoint=0, duration=10):
+        """Creates a clip on the specified layer."""
         asset = GES.UriClipAsset.request_sync(
             common.get_sample_uri("tears_of_steel.webm"))
-        clips = [layer.add_asset(asset, i * 10, 0, 10, GES.TrackType.UNKNOWN)
-                 for i in range(num_clips)]
+        return layer.add_asset(asset, start, inpoint, duration, GES.TrackType.UNKNOWN)
+
+    def addClipsSimple(self, timeline, num_clips):
+        """Creates a number of clips on a new layer."""
+        layer = timeline.ges_timeline.append_layer()
+        clips = [self.add_clip(layer, i * 10) for i in range(num_clips)]
         self.assertEqual(len(clips), num_clips)
         return clips
 
 
 class TestLayers(BaseTestTimeline):
+    """Tests for the layers."""
 
     def testDraggingLayer(self):
         self.checkGetLayerAt([THIN, THIN, THIN], 1, True,
@@ -82,7 +88,7 @@ class TestLayers(BaseTestTimeline):
         # Allocate layers
         y = 0
         for priority, height in enumerate(heights):
-            ges_layer = timeline.createLayer(priority=priority)
+            ges_layer = timeline.create_layer(priority=priority)
             rect = Gdk.Rectangle()
             rect.y = y
             rect.height = height
@@ -153,7 +159,7 @@ class TestLayers(BaseTestTimeline):
         timeline = create_timeline_container().timeline
         ges_layers = []
         for priority in start_priorities:
-            ges_layer = timeline.createLayer(priority)
+            ges_layer = timeline.create_layer(priority)
             self.assertEqual(ges_layer.props.priority, priority)
             ges_layers.append(ges_layer)
         self.check_priorities_and_positions(timeline, ges_layers, expected_priorities)
@@ -196,27 +202,29 @@ class TestLayers(BaseTestTimeline):
         self.assertListEqual(positions, expected_positions)
 
     def test_remove_layer(self):
-        self.check_remove_layer([0, 0, 0, 0])
-        self.check_remove_layer([0, 0, 1, 0])
-        self.check_remove_layer([0, 1, 0, 0])
-        self.check_remove_layer([0, 2, 1, 0])
-        self.check_remove_layer([1, 0, 1, 0])
-        self.check_remove_layer([2, 2, 0, 0])
-        self.check_remove_layer([3, 2, 1, 0])
+        self.check_remove_layer([0, 0, 0])
+        self.check_remove_layer([0, 0, 1])
+        self.check_remove_layer([0, 1, 0])
+        self.check_remove_layer([0, 2, 1])
+        self.check_remove_layer([1, 0, 1])
+        self.check_remove_layer([2, 2, 0])
+        self.check_remove_layer([3, 2, 1])
 
     def check_remove_layer(self, removal_order):
         timeline = create_timeline_container().timeline
 
         # Add layers to remove them later.
         ges_layers = []
-        for priority in range(len(removal_order)):
-            ges_layer = timeline.createLayer(priority)
+        # Pitivi doesn't support removing the last remaining layer,
+        # that's why we create an extra layer.
+        for priority in range(len(removal_order) + 1):
+            ges_layer = timeline.create_layer(priority)
             ges_layers.append(ges_layer)
 
-        # Remove the layers in the specified order.
+        # Remove layers one by one in the specified order.
         for priority in removal_order:
             ges_layer = ges_layers[priority]
-            self.assertTrue(timeline.ges_timeline.remove_layer(ges_layer))
+            ges_layer.control_ui.delete_layer_action.activate(None)
             ges_layers.remove(ges_layer)
             self.check_priorities_and_positions(timeline, ges_layers, list(range(len(ges_layers))))
 
@@ -235,7 +243,7 @@ class TestLayers(BaseTestTimeline):
         # Add layers to move them later.
         ges_layers = []
         for priority in range(len(expected_priorities)):
-            ges_layer = timeline.createLayer(priority)
+            ges_layer = timeline.create_layer(priority)
             ges_layers.append(ges_layer)
 
         timeline.moveLayer(ges_layers[from_priority], to_priority)
