@@ -18,6 +18,7 @@
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 from unittest import mock
+from unittest import skip
 from unittest import TestCase
 
 from gi.repository import Gdk
@@ -32,7 +33,7 @@ from pitivi.timeline.timeline import TimelineContainer
 from pitivi.undo.project import AssetAddedAction
 from pitivi.undo.timeline import ClipAdded
 from pitivi.undo.timeline import ClipRemoved
-from pitivi.undo.timeline import EffectAddedAction
+from pitivi.undo.timeline import TrackElementAdded
 from pitivi.undo.undo import PropertyChangedAction
 from pitivi.utils.ui import LAYER_HEIGHT
 from pitivi.utils.ui import URI_TARGET_ENTRY
@@ -78,6 +79,26 @@ class BaseTestUndoTimeline(TestCase):
         mainloop.run()
         self.assertTrue(self.timeline.props.auto_transition)
 
+    def assert_effect_count(self, clip, count):
+        effects = [effect for effect in clip.get_children(True)
+                   if isinstance(effect, GES.Effect)]
+        self.assertEqual(len(effects), count)
+
+    def get_transition_element(self, ges_layer):
+        """"Gets the first found GES.VideoTransition clip."""
+        for clip in ges_layer.get_clips():
+            if isinstance(clip, GES.TransitionClip):
+                for element in clip.get_children(False):
+                    if isinstance(element, GES.VideoTransition):
+                        return element
+
+    def check_layers(self, layers):
+        self.assertEqual(self.timeline.get_layers(), layers)
+        # Import TestLayers locally, otherwise its tests are discovered and
+        # run twice.
+        from tests.test_timeline_timeline import TestLayers
+        TestLayers.check_priorities_and_positions(self, self.timeline.ui, layers, list(range(len(layers))))
+
 
 class TestTimelineObserver(BaseTestUndoTimeline):
 
@@ -87,26 +108,30 @@ class TestTimelineObserver(BaseTestUndoTimeline):
         layer1 = self.layer
         layer2 = self.timeline.append_layer()
         layer3 = self.timeline.append_layer()
+        self.check_layers([layer1, layer2, layer3])
+        self.check_removal(self.timeline.get_layers())
 
-        self.assertEqual([layer1, layer2, layer3], self.timeline.get_layers())
-        self.assertEqual([l.props.priority for l in [layer1, layer2, layer3]],
-                         list(range(3)))
+    def check_removal(self, ges_layers):
+        if len(ges_layers) == 1:
+            # We don't support removing the last remaining layer.
+            return
+        for ges_layer in ges_layers:
+            remaining_layers = list(ges_layers)
+            remaining_layers.remove(ges_layer)
 
-        with self.action_log.started("layer removed"):
-            self.timeline.remove_layer(layer2)
+            ges_layer.control_ui.delete_layer_action.activate(None)
+            self.check_layers(remaining_layers)
 
-        self.assertEqual([layer1, layer3], self.timeline.get_layers())
-        self.assertEqual([l.props.priority for l in [layer1, layer3]],
-                         list(range(2)))
+            self.action_log.undo()
+            self.check_layers(ges_layers)
 
-        self.action_log.undo()
-        self.assertEqual([layer1, layer2, layer3], self.timeline.get_layers())
-        self.assertEqual([l.props.priority for l in [layer1, layer2, layer3]],
-                         list(range(3)))
-        self.action_log.redo()
-        self.assertEqual([layer1, layer3], self.timeline.get_layers())
-        self.assertEqual([l.props.priority for l in [layer1, layer3]],
-                         list(range(2)))
+            self.action_log.redo()
+            self.check_layers(remaining_layers)
+
+            self.check_removal(remaining_layers)
+
+            self.action_log.undo()
+            self.check_layers(ges_layers)
 
     def test_group_ungroup_clips(self):
         self.setup_timeline_container()
@@ -153,6 +178,105 @@ class TestTimelineObserver(BaseTestUndoTimeline):
             self.assertIsNone(clip1.get_parent())
             self.assertIsNone(clip2.get_parent())
 
+    def test_ungroup_group_clip(self):
+        self.setup_timeline_container()
+        timeline = self.timeline_container.timeline
+
+        uri = common.get_sample_uri("tears_of_steel.webm")
+        asset = GES.UriClipAsset.request_sync(uri)
+        clip = asset.extract()
+        self.layer.add_clip(clip)
+        clips = list(self.getTimelineClips())
+        self.assertEqual(len(clips), 1, clips)
+        self.assertEqual(len(clips[0].get_children(False)), 2)
+
+        timeline.selection.select([clip])
+        timeline.resetSelectionGroup()
+        timeline.current_group.add(clip)
+        self.timeline_container.ungroup_action.activate(None)
+        clips = list(self.getTimelineClips())
+        self.assertEqual(len(clips), 2, clips)
+        self.assertEqual(len(clips[0].get_children(False)), 1)
+        self.assertEqual(len(clips[1].get_children(False)), 1)
+
+        timeline.selection.select(clips)
+        timeline.resetSelectionGroup()
+        for clip in clips:
+            timeline.current_group.add(clip)
+        self.timeline_container.group_action.activate(None)
+        clips = list(self.getTimelineClips())
+        self.assertEqual(len(clips), 1, clips)
+        self.assertEqual(len(clips[0].get_children(False)), 2)
+
+        for i in range(2):
+            # Undo grouping.
+            self.action_log.undo()
+            clips = list(self.getTimelineClips())
+            self.assertEqual(len(clips), 2, clips)
+            self.assertEqual(len(clips[0].get_children(False)), 1)
+            self.assertEqual(len(clips[1].get_children(False)), 1)
+
+            # Undo ungrouping.
+            self.action_log.undo()
+            clips = list(self.getTimelineClips())
+            self.assertEqual(len(clips), 1, clips)
+            self.assertEqual(len(clips[0].get_children(False)), 2)
+
+            # Redo ungrouping.
+            self.action_log.redo()
+            clips = list(self.getTimelineClips())
+            self.assertEqual(len(clips), 2, clips)
+            self.assertEqual(len(clips[0].get_children(False)), 1)
+            self.assertEqual(len(clips[1].get_children(False)), 1)
+
+            # Redo grouping.
+            self.action_log.redo()
+            clips = list(self.getTimelineClips())
+            self.assertEqual(len(clips), 1, clips)
+            self.assertEqual(len(clips[0].get_children(False)), 2)
+
+    def test_insert_on_first_layer(self):
+        self.setup_timeline_container()
+        uri = common.get_sample_uri("tears_of_steel.webm")
+        asset = GES.UriClipAsset.request_sync(uri)
+        clip1 = asset.extract()
+        self.timeline_container.insert_clips_on_first_layer(clips=[clip1], position=0)
+        clips = list(self.getTimelineClips())
+        self.assertEqual(len(clips), 1, clips)
+
+        # Undo insert on first layer
+        self.action_log.undo()
+        clips = list(self.getTimelineClips())
+        self.assertEqual(len(clips), 0, clips)
+
+        # Redo insert on first layer
+        self.action_log.redo()
+        clips = list(self.getTimelineClips())
+        self.assertEqual(len(clips), 1, clips)
+
+        # Insert new clip to create a layer
+        clip2 = common.create_test_clip(GES.TitleClip)
+        clip2.set_start(0 * Gst.SECOND)
+        clip2.set_duration(1 * Gst.SECOND)
+        self.timeline_container.insert_clips_on_first_layer(clips=[clip2], position=0)
+        layers = self.timeline.get_layers()
+        self.check_layers([layers[0], self.layer])
+        self.assertEqual(layers[0].get_clips(), [clip2])
+        self.assertEqual(layers[1].get_clips(), [clip1])
+
+        # Undo insert to create a layer
+        self.action_log.undo()
+        layers = self.timeline.get_layers()
+        self.check_layers([self.layer])
+        self.assertEqual(layers[0].get_clips(), [clip1])
+
+        # Redo insert to create a layer
+        self.action_log.redo()
+        layers = self.timeline.get_layers()
+        self.check_layers([layers[0], self.layer])
+        self.assertEqual(layers[0].get_clips(), [clip2])
+        self.assertEqual(layers[1].get_clips(), [clip1])
+
 
 class TestLayerObserver(BaseTestUndoTimeline):
 
@@ -180,13 +304,13 @@ class TestLayerObserver(BaseTestUndoTimeline):
                     timeline_ui._motion_notify_event_cb(None, event=event)
 
             timeline_ui._button_release_event_cb(None, event=event)
-        self.assertEqual(self.timeline.get_layers(), [layer2, layer3, layer1])
+        self.check_layers([layer2, layer3, layer1])
 
         self.action_log.undo()
-        self.assertEqual(self.timeline.get_layers(), [layer1, layer2, layer3])
+        self.check_layers([layer1, layer2, layer3])
 
         self.action_log.redo()
-        self.assertEqual(self.timeline.get_layers(), [layer2, layer3, layer1])
+        self.check_layers([layer2, layer3, layer1])
 
     def test_layer_renamed(self):
         layer = Layer(self.layer, timeline=mock.Mock())
@@ -208,7 +332,7 @@ class TestLayerObserver(BaseTestUndoTimeline):
             self.layer.add_clip(clip1)
 
         stack, = self.action_log.undo_stacks
-        self.assertEqual(len(stack.done_actions), 1, stack.done_actions)
+        self.assertEqual(len(stack.done_actions), 2, stack.done_actions)
         self.assertTrue(isinstance(stack.done_actions[0], ClipAdded))
         self.assertTrue(clip1 in self.getTimelineClips())
 
@@ -319,7 +443,7 @@ class TestLayerObserver(BaseTestUndoTimeline):
         stack = stacks[0]
         self.assertEqual(1, len(stack.done_actions), stack.done_actions)
         action = stack.done_actions[0]
-        self.assertTrue(isinstance(action, EffectAddedAction))
+        self.assertTrue(isinstance(action, TrackElementAdded))
 
         self.assertTrue(effect1 in clip1.get_children(True))
         self.assertEqual(1, len([effect for effect in
@@ -349,7 +473,7 @@ class TestLayerObserver(BaseTestUndoTimeline):
         stack = stacks[0]
         self.assertEqual(1, len(stack.done_actions), stack.done_actions)
         action = stack.done_actions[0]
-        self.assertTrue(isinstance(action, EffectAddedAction))
+        self.assertTrue(isinstance(action, TrackElementAdded))
 
         self.assertTrue(effect1 in clip1.get_children(True))
         self.assertEqual(1, len([effect for effect in
@@ -358,20 +482,13 @@ class TestLayerObserver(BaseTestUndoTimeline):
 
         with self.action_log.started("remove effect"):
             clip1.remove(effect1)
-
-        self.assertEqual(0, len([effect for effect in
-                                 clip1.get_children(True)
-                                 if isinstance(effect, GES.Effect)]))
+        self.assert_effect_count(clip1, 0)
 
         self.action_log.undo()
-        self.assertEqual(1, len([effect for effect in
-                                 clip1.get_children(True)
-                                 if isinstance(effect, GES.Effect)]))
+        self.assert_effect_count(clip1, 1)
 
         self.action_log.redo()
-        self.assertEqual(0, len([effect for effect in
-                                 clip1.get_children(True)
-                                 if isinstance(effect, GES.Effect)]))
+        self.assert_effect_count(clip1, 0)
 
     def test_move_clip(self):
         self._wait_until_project_loaded()
@@ -401,7 +518,8 @@ class TestLayerObserver(BaseTestUndoTimeline):
         self.assertEqual(clip2.get_start(), 20 * Gst.SECOND)
         self.assertEqual(len(self.layer.get_clips()), 2)
 
-    def test_transitions(self):
+    def test_transition_type(self):
+        """Checks the transitions keep their type."""
         self._wait_until_project_loaded()
         uri = common.get_sample_uri("tears_of_steel.webm")
         asset = GES.UriClipAsset.request_sync(uri)
@@ -422,15 +540,7 @@ class TestLayerObserver(BaseTestUndoTimeline):
             self.assertNotIsInstance(action, AssetAddedAction,
                                      stack.done_actions)
 
-        def get_transition_element(ges_layer):
-            for clip in ges_layer.get_clips():
-                if isinstance(clip, GES.TransitionClip):
-                    for element in clip.get_children(False):
-                        if isinstance(element, GES.VideoTransition):
-                            return element
-            self.fail("Cannot find video transition")
-
-        transition_element = get_transition_element(self.layer)
+        transition_element = self.get_transition_element(self.layer)
         self.assertEqual(transition_element.get_transition_type(),
                          GES.VideoStandardTransitionType.CROSSFADE)
 
@@ -439,11 +549,12 @@ class TestLayerObserver(BaseTestUndoTimeline):
         self.assertEqual(transition_element.get_transition_type(),
                          GES.VideoStandardTransitionType.BAR_WIPE_LR)
 
-        # Try to undo/redo
+        # Undo setting the transition type.
         self.action_log.undo()
         self.assertEqual(transition_element.get_transition_type(),
                          GES.VideoStandardTransitionType.CROSSFADE)
 
+        # Redo setting the transition type.
         self.action_log.redo()
         self.assertEqual(transition_element.get_transition_type(),
                          GES.VideoStandardTransitionType.BAR_WIPE_LR)
@@ -454,19 +565,19 @@ class TestLayerObserver(BaseTestUndoTimeline):
             with self.action_log.started("remove clip"):
                 self.layer.remove_clip(clip2)
             self.action_log.undo()
-            transition_element = get_transition_element(self.layer)
+            transition_element = self.get_transition_element(self.layer)
             self.assertEqual(transition_element.get_transition_type(),
                              GES.VideoStandardTransitionType.BAR_WIPE_LR)
 
             # Undo a transition change operation done on a now obsolete
             # transition clip.
             self.action_log.undo()
-            transition_element = get_transition_element(self.layer)
+            transition_element = self.get_transition_element(self.layer)
             self.assertEqual(transition_element.get_transition_type(),
                              GES.VideoStandardTransitionType.CROSSFADE)
 
             self.action_log.redo()
-            transition_element = get_transition_element(self.layer)
+            transition_element = self.get_transition_element(self.layer)
             self.assertEqual(transition_element.get_transition_type(),
                              GES.VideoStandardTransitionType.BAR_WIPE_LR,
                              "The auto objects map in "
@@ -480,18 +591,76 @@ class TestLayerObserver(BaseTestUndoTimeline):
             self.action_log.undo()
             # Redo adding the second clip.
             self.action_log.redo()
-            transition_element = get_transition_element(self.layer)
+            transition_element = self.get_transition_element(self.layer)
             self.assertEqual(transition_element.get_transition_type(),
                              GES.VideoStandardTransitionType.CROSSFADE)
             # Redo the transition change.
             self.action_log.redo()
-            transition_element = get_transition_element(self.layer)
+            transition_element = self.get_transition_element(self.layer)
             self.assertEqual(transition_element.get_transition_type(),
                              GES.VideoStandardTransitionType.BAR_WIPE_LR,
                              "The auto objects map in "
                              "UndoableAutomaticObjectAction is not updated when "
                              "redoing clip add.")
 
+    def test_transition_found(self):
+        self._wait_until_project_loaded()
+        uri = common.get_sample_uri("1sec_simpsons_trailer.mp4")
+        asset = GES.UriClipAsset.request_sync(uri)
+
+        clip1 = asset.extract()
+        clip1.set_start(0)
+        self.layer.add_clip(clip1)
+
+        clip2 = asset.extract()
+        clip2.set_start(clip1.props.duration)
+        self.layer.add_clip(clip2)
+
+        self.assertIsNone(self.get_transition_element(self.layer))
+        with self.action_log.started("move clip1"):
+            clip1.edit([], -1, GES.EditMode.EDIT_NORMAL, GES.Edge.EDGE_NONE, clip1.props.duration / 2)
+        self.assertIsNotNone(self.get_transition_element(self.layer))
+
+        self.action_log.undo()
+        self.assertIsNone(self.get_transition_element(self.layer))
+
+        self.action_log.redo()
+        self.assertIsNotNone(self.get_transition_element(self.layer))
+
+    def test_paste_undo(self):
+        """Checks a paste operation can be undone."""
+        self.setup_timeline_container()
+        timeline = self.timeline_container.timeline
+        project = timeline.ges_timeline.get_asset()
+
+        # Create test clip
+        clip = common.create_test_clip(GES.TitleClip)
+        clip.props.start = 0
+        clip.props.duration = 10
+        self.layer.add_clip(clip)
+        self.assertEqual(len(self.layer.get_clips()), 1)
+
+        # Select the test clip
+        event = mock.Mock()
+        event.get_button.return_value = (True, 1)
+        with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
+            get_event_widget.return_value = clip.ui
+            clip.ui.timeline._button_press_event_cb(None, event)
+        clip.ui._button_release_event_cb(None, event)
+
+        self.timeline_container.copy_action.emit("activate", None)
+
+        position = 10
+        project.pipeline.getPosition = mock.Mock(return_value=position)
+        self.timeline_container.paste_action.emit("activate", None)
+        self.assertEqual(len(self.layer.get_clips()), 2)
+
+        self.action_log.undo()
+        self.assertEqual(len(self.layer.get_clips()), 1)
+
+        self.action_log.redo()
+        self.assertEqual(len(self.layer.get_clips()), 2)
+
 
 class TestControlSourceObserver(BaseTestUndoTimeline):
 
@@ -598,7 +767,7 @@ class TestTimelineElementObserver(BaseTestUndoTimeline):
         stack = stacks[0]
         self.assertEqual(1, len(stack.done_actions), stack.done_actions)
         action = stack.done_actions[0]
-        self.assertTrue(isinstance(action, EffectAddedAction))
+        self.assertTrue(isinstance(action, TrackElementAdded))
 
         self.assertTrue(effect1 in clip1.get_children(True))
         self.assertEqual(1, len([effect for effect in
@@ -673,10 +842,57 @@ class TestGObjectObserver(BaseTestUndoTimeline):
         self.action_log.redo()
         self.assertEqual(20, clip1.get_priority())
 
+    def test_effect_toggling(self):
+        clip1 = GES.TitleClip()
+        self.layer.add_clip(clip1)
+
+        effect1 = GES.Effect.new("agingtv")
+        with self.action_log.started("add effect"):
+            clip1.add(effect1)
+        self.assertTrue(effect1.props.active)
+        self.assert_effect_count(clip1, 1)
+
+        with self.action_log.started("toggle effect"):
+            effect1.props.active = False
+        self.assertFalse(effect1.props.active)
+
+        with self.action_log.started("remove effect"):
+            clip1.remove(effect1)
+        self.assert_effect_count(clip1, 0)
+
+        # Undo effect removing.
+        self.action_log.undo()
+        self.assert_effect_count(clip1, 1)
+
+        # Undo effect toggling.
+        self.action_log.undo()
+        self.assertTrue(effect1.props.active)
+
+        # Redo effect toggling.
+        self.action_log.redo()
+        self.assertFalse(effect1.props.active)
+
+        # Undo effect toggling.
+        self.action_log.undo()
+        self.assertTrue(effect1.props.active)
+
+        # Undo effect add.
+        self.action_log.undo()
+        self.assertFalse(effect1 in clip1.get_children(True))
+
+        # Redo effect add.
+        self.action_log.redo()
+        self.assertTrue(effect1 in clip1.get_children(True))
+        self.assertTrue(effect1.props.active)
+
+        # Redo effect toggling.
+        self.action_log.redo()
+        self.assertFalse(effect1.props.active)
+
 
 class TestDragDropUndo(BaseTestUndoTimeline):
 
-    def test_clip_dragged_to_create_layer(self):
+    def test_clip_dragged_to_create_layer_below(self):
         self.setup_timeline_container()
         timeline_ui = self.timeline_container.timeline
         layers = self.timeline.get_layers()
@@ -709,6 +925,7 @@ class TestDragDropUndo(BaseTestUndoTimeline):
         layers = self.timeline.get_layers()
         self.assertEqual(len(layers), 2)
         self.assertEqual(layers[0], self.layer)
+        self.check_layers(layers)
         self.assertEqual(layers[0].get_clips(), [])
         self.assertEqual(layers[1].get_clips(), [clip])
 
@@ -716,15 +933,68 @@ class TestDragDropUndo(BaseTestUndoTimeline):
         layers = self.timeline.get_layers()
         self.assertEqual(len(layers), 1)
         self.assertEqual(layers[0], self.layer)
+        self.check_layers(layers)
         self.assertEqual(layers[0].get_clips(), [clip])
 
         self.action_log.redo()
         layers = self.timeline.get_layers()
         self.assertEqual(len(layers), 2)
         self.assertEqual(layers[0], self.layer)
+        self.check_layers(layers)
         self.assertEqual(layers[0].get_clips(), [])
         self.assertEqual(layers[1].get_clips(), [clip])
 
+    def test_clip_dragged_to_create_layer_above(self):
+        self.setup_timeline_container()
+        timeline_ui = self.timeline_container.timeline
+        layers = self.timeline.get_layers()
+        self.assertEqual(len(layers), 1)
+
+        clip = GES.TitleClip()
+        self.layer.add_clip(clip)
+
+        # Drag a clip on a separator to create a layer.
+        with mock.patch.object(Gtk, 'get_event_widget') as get_event_widget:
+            get_event_widget.return_value = clip.ui
+
+            event = mock.Mock()
+            event.x = 0
+            event.get_button.return_value = True, 1
+            timeline_ui._button_press_event_cb(None, event)
+
+            def translate_coordinates(widget, x, y):
+                return x, y
+            clip.ui.translate_coordinates = translate_coordinates
+            event = mock.Mock()
+            event.get_state.return_value = Gdk.ModifierType.BUTTON1_MASK
+            event.x = 1
+            event.y = -1
+            event.get_button.return_value = True, 1
+            timeline_ui._motion_notify_event_cb(None, event)
+
+        timeline_ui._button_release_event_cb(None, event)
+
+        layers = self.timeline.get_layers()
+        self.assertEqual(len(layers), 2)
+        self.assertEqual(layers[1], self.layer)
+        self.check_layers(layers)
+        self.assertEqual(layers[0].get_clips(), [clip])
+        self.assertEqual(layers[1].get_clips(), [])
+
+        self.action_log.undo()
+        layers = self.timeline.get_layers()
+        self.assertEqual(len(layers), 1)
+        self.check_layers(layers)
+        self.assertEqual(layers[0].get_clips(), [clip])
+
+        self.action_log.redo()
+        layers = self.timeline.get_layers()
+        self.assertEqual(len(layers), 2)
+        self.assertEqual(layers[1], self.layer)
+        self.check_layers(layers)
+        self.assertEqual(layers[0].get_clips(), [clip])
+        self.assertEqual(layers[1].get_clips(), [])
+
     def test_media_library_asset_dragged_on_separator(self):
         """Simulate dragging an asset from the media library to the timeline."""
         self.setup_timeline_container()
